@@ -1,25 +1,27 @@
 import os
-import pathlib
+from  pathlib import Path
 from typing import Set, List
 
 
-def get_caller_file(ignore_callers: Set[str] = None):
+def get_caller_file(ignore_callers: Set[str] = None) -> str:
     if ignore_callers is None:
         ignore_callers = {}
 
     import inspect
 
     frames: List[inspect.FrameInfo] = inspect.stack()
-    lab_src = pathlib.PurePath(__file__).parent.parent
+    lab_src = Path(__file__).absolute().parent.parent
 
     for f in frames:
-        module_path = pathlib.PurePath(f.filename)
+        module_path = Path(f.filename).absolute()
         if str(module_path).startswith(str(lab_src) + '/'):
             continue
+        if str(module_path).find('/dist-packages/') != -1:
+            continue
         if str(module_path) in ignore_callers:
             continue
         if str(module_path).startswith('<ipython'):
             break
         return str(module_path)
 
-    return os.path.abspath('')
\ No newline at end of file
+    return str(Path('').absolute())
