@@ -1,7 +1,4 @@
 """
-[![PiPy Version](https://badge.fury.io/py/labml-nn.svg)](https://badge.fury.io/py/labml-nn)
-[![PiPy Downloads](https://pepy.tech/badge/labml-nn)](https://pepy.tech/project/labml-nn)
-
 # [LabML Neural Networks](https://lab-ml.com/labml_nn/index.html)
 
 This is a collection of simple PyTorch implementations of
